import { AbstractTable } from 'drizzle-orm';
import { createEnum } from 'drizzle-orm/types/type';

export class CitiesTable2 extends AbstractTable<CitiesTable2> {
    public name = this.timestamp('name', { notNull: true }).defaultValue(new Date());
    public page = this.varchar('page', { size: 256 });
    public roleEnum = this.type(rolesEnumMulti, 'role')

    public userId1 = this.int('user_id').foreignKey(UsersTable2, (table) => table.id);

    public data = this.jsonb<string[]>('data');

    public tableName(): string {
        return 'citiess';
    }
}

export class UsersTable2 extends AbstractTable<UsersTable2> {
    public id = this.int('id').autoIncrement().primaryKey();
    public phone = this.varchar('phone', { size: 256 }).unique();
    public fullName = this.varchar('full_name', { size: 512 });
    public test = this.decimal('test', { notNull: true, precision: 100, scale: 2 });
    public test1 = this.bigint('test1', { notNull: true });
    public createdAt = this.timestamp('created_at', { notNull: true });
    public updatedAt = this.timestamp('updated_at', { notNull: true });

    public phoneFullNameIndex = this.index([this.phone, this.fullName]);
    public phoneIndex = this.index(this.phone);

    public tableName(): string {
        return 'users';
    }
}

export const rolesEnumMulti = createEnum({
    alias: "roles_enum",
    values: ["role1", "role2", "role3"],
});

export const rolesEnumMulti2 = createEnum({
    alias: "roles2_enum",
    values: ["role1.1", "role2.2", "role3.3"],
});
