import ts from "typescript";
import fs from "fs";
const printer: ts.Printer = ts.createPrinter();

const prepareFabricFile = (folder: string) => {
  const staticImports = [
    ts.createImportDeclaration(
      undefined,
      undefined,
      ts.createImportClause(
        undefined,
        ts.createNamedImports([
          ts.createImportSpecifier(
            undefined,
            ts.createIdentifier("AbstractTable"),
          ),
          ts.createImportSpecifier(undefined, ts.createIdentifier("DB")),
        ]),
      ),
      ts.createStringLiteral("@lambda-team/ltdl"),
    ),
    ts.createImportDeclaration(
      undefined,
      undefined,
      ts.createImportClause(
        undefined,
        ts.createNamedImports([
          ts.createImportSpecifier(undefined, ts.createIdentifier("Pool")),
        ]),
      ),
      ts.createStringLiteral("pg"),
    ),
    ts.createImportDeclaration(
      undefined,
      undefined,
      ts.createImportClause(ts.createIdentifier("Serializer"), undefined),
      ts.createStringLiteral("./serializer"),
    ),
  ];

  const dynamicImports = [];
  const filenames = fs.readdirSync(folder);
  for (let i = 0; i < filenames.length; i++) {
    const filename = filenames[i];
    const importPath = `${folder}${filename.split(".")[0]}`;
    dynamicImports.push(ts.createImportDeclaration(
      undefined,
      undefined,
      ts.createImportClause(
        undefined,
        ts.createNamespaceImport(ts.createIdentifier(`i${i}`)),
      ),
      ts.createStringLiteral(importPath),
    ));
  }

  const variablesStatements = [
    ts.createVariableStatement(
      undefined,
      ts.createVariableDeclarationList(
        [
          ts.createVariableDeclaration(
            ts.createIdentifier("db"),
            undefined,
            ts.createNew(ts.createIdentifier("DB"), undefined, [
              ts.createNew(ts.createIdentifier("Pool"), undefined, []),
            ]),
          ),
        ],
        ts.NodeFlags.Const,
      ),
    ),
    ts.createVariableStatement(
      undefined,
      ts.createVariableDeclarationList(
        [
          ts.createVariableDeclaration(
            ts.createIdentifier("serializer"),
            undefined,
            ts.createNew(ts.createIdentifier("Serializer"), undefined, []),
          ),
        ],
        ts.NodeFlags.Const,
      ),
    ),
  ];

  const blockStatements = [];

  // const tables = []
  blockStatements.push(ts.createVariableStatement(
    undefined,
    ts.createVariableDeclarationList(
      [
        ts.createVariableDeclaration(
          ts.createIdentifier("tables"),
          undefined,
          ts.createArrayLiteral([], false),
        ),
      ],
      ts.NodeFlags.Const,
    ),
  ));
  for (let i = 0; i < filenames.length; i++) {
    // const t1 = (new i1.default(db) as unknown as AbstractTable<any>);
    // tables.push(t1)
    const blockStatement = [
      ts.createVariableStatement(
        undefined,
        ts.createVariableDeclarationList(
          [
            ts.createVariableDeclaration(
              ts.createIdentifier("t" + i),
              undefined,
              ts.createParen(
                ts.createAsExpression(
                  ts.createAsExpression(
                    ts.createNew(
                      ts.createPropertyAccess(
                        ts.createIdentifier("i" + i),
                        ts.createIdentifier("default"),
                      ),
                      undefined,
                      [ts.createIdentifier("db")],
                    ),
                    ts.createKeywordTypeNode(
                      ts.SyntaxKind.UnknownKeyword,
                    ),
                  ),
                  ts.createTypeReferenceNode(
                    ts.createIdentifier("AbstractTable"),
                    [
                      ts.createKeywordTypeNode(
                        ts.SyntaxKind.AnyKeyword,
                      ),
                    ],
                  ),
                ),
              ),
            ),
          ],
          ts.NodeFlags.Const,
        ),
      ),
      ts.createExpressionStatement(
        ts.createCall(
          ts.createPropertyAccess(
            ts.createIdentifier("tables"),
            ts.createIdentifier("push"),
          ),
          undefined,
          [ts.createIdentifier("t" + i)],
        ),
      ),
    ];

    blockStatements.push(...blockStatement);
  }

  // return serializer.generate(tables)
  blockStatements.push(ts.createReturn(
    ts.createCall(
      ts.createPropertyAccess(
        ts.createIdentifier("serializer"),
        ts.createIdentifier("generate"),
      ),
      undefined,
      [ts.createIdentifier("tables")],
    ),
  ));

  const funcStatement = [
    ts.createVariableStatement(
      undefined,
      ts.createVariableDeclarationList(
        [
          ts.createVariableDeclaration(
            ts.createIdentifier("testFun"),
            undefined,
            ts.createArrowFunction(
              undefined,
              undefined,
              [],
              undefined,
              ts.createToken(ts.SyntaxKind.EqualsGreaterThanToken),
              // function body
              ts.createBlock(
                blockStatements,
                true,
              ),
            ),
          ),
        ],
        ts.NodeFlags.Const,
      ),
    ),
  ];
  const invocationStatement = [
    ts.createExpressionStatement(
      ts.createCall(ts.createIdentifier("testFun"), undefined, []),
    ),
  ];

  const outFile: ts.SourceFile = ts.createSourceFile(
    "outfile.ts",
    "",
    ts.ScriptTarget.ES2015,
    true,
    ts.ScriptKind.TS,
  );

  const source = [];
  source.push(...staticImports);
  source.push(...dynamicImports);
  source.push(...variablesStatements);
  source.push(...funcStatement);
  source.push(...invocationStatement);

  const newFile = ts.factory.updateSourceFile(outFile, source);

  return printer.printFile(newFile);
};

export default prepareFabricFile;