/* eslint-disable no-restricted-syntax */
/* eslint-disable new-cap */
/* eslint-disable global-require */
/* eslint-disable import/no-dynamic-require */
import { AbstractTable, Column, DB } from 'drizzle-orm';
import ColumnType from 'drizzle-orm/columns/types/columnType';
import PgEnum from 'drizzle-orm/columns/types/pgEnum';
import TableIndex from 'drizzle-orm/indexes/tableIndex';
import Enum from 'drizzle-orm/types/type';
import { Pool } from 'pg';

interface ColumnAsObject {
  [name: string]: {
    name?: string;
    type?: string;
    primaryKey?: boolean;
    autoincrement?: boolean;
    unique?: boolean;
    default?: any;
    notNull?: boolean;
    references?: {
      table: string;
      column: string;
    };
  };
}

interface IndexColumnAsObject {
  [name: string]: {
    name?: string;
  };
}

interface IndexAsObject {
  [name: string]: {
    name?: string;
    columns?: ColumnAsObject;
  };
}

interface TableAsObject {
  [name: string]: {
    name: string;
    columns: ColumnAsObject;
    indexes: {
      [name: string]: {
        name?: string;
        type?: string;
      };
    };
  };
}

interface EnumAsObject {
  [name: string]: {
    name: string;
    values: string[];
  };
}

export default class MigrationSerializer {
  db = new DB(new Pool());

  public generate = (tables: AbstractTable<any>[], enums: Enum<any>[]) => {
    const result: TableAsObject = {};

    for (const table of tables) {
      const tableEntries = Object.entries(table);
      const columnToReturn: ColumnAsObject = {};
      const indexToReturn: IndexAsObject = {};

      for (const properties of tableEntries) {
        const key = properties[0];
        const value = properties[1];
        if (value instanceof TableIndex) {
          const columns = value.getColumns();
          const name = value.indexName();

          const indexColumnToReturn: IndexColumnAsObject = {};

          for (const column of columns) {
            const columnName = column.getColumnName();
            indexColumnToReturn[columnName] = {
              name: columnName,
            };
          }

          indexToReturn[name] = {
            name,
            columns: indexColumnToReturn,
          };
        }

        if (value instanceof Column) {
          columnToReturn[key] = {
            name: value.getColumnName(),
            type: (value.getColumnType() as ColumnType).getDbName(),
            primaryKey: !!value.primaryKeyName,
            autoincrement: value.isAutoIncrement(),
            unique: !!value.uniqueKeyName,
            default: value.getDefaultValue() === null ? undefined : value.getDefaultValue(),
            notNull: !value.isNullableFlag,
          };

          const referenced = value.getReferenced();
          if (referenced) {
            columnToReturn[key].references = {
              table: referenced.getParentName(),
              column: referenced.getColumnName(),
            };
          }
        }
      }

      result[table.tableName()] = {
        name: table.tableName(),
        columns: columnToReturn,
        indexes: indexToReturn,
      };
    }

    // const enumsToReturn = enums.map((it) => {
    //   return { name: it.name, values: it.values }
    // });

    const enumsToReturn = enums.reduce<{[key:string]: Enum<any>}>((map, obj) => {
      const key = obj.name;
      const newValues = obj.values.reduce((mapped, value) => {
        mapped[value] = value;
        return mapped;
      }, {});

      map[key] = { name: obj.name, values: newValues };
      return map;
    }, {});

    return { version: '1', tables: result, enums: enumsToReturn };
  };
}
