import React, { useEffect, useState } from 'react';
import { Box, Newline, Text } from 'ink';
import Spinner from 'ink-spinner';
import { CreateTaskParams } from 'cli/components-api/index';

enum StepState {
  IN_PROGRESS,
  SUCCESS,
  FAIL
}

export interface CallbackProps {
  setTitle: React.Dispatch<React.SetStateAction<string>>
  setError: React.Dispatch<React.SetStateAction<any>>
}

interface Props<T> {
  func: (props: CallbackProps) => T;
  onDone: (result: T) => T;
  titleStr: string;
}

const Task: React.FC<Props<any>> = <T extends unknown>(props: Props<T>) => {
  const {
    func,
    onDone,
    titleStr,
  } = props;
  const [title, setTitle] = useState<string>(titleStr);
  const [state, setState] = useState<StepState>(StepState.IN_PROGRESS);
  const [error, setError] = useState<any>(null);

  useEffect(() => {
    (async () => {
      try {
        const result = await func({ setTitle, setError });
        setState(StepState.SUCCESS);
        onDone(result);
      } catch (e) {
        setError(e);
        setState(StepState.FAIL);
      }
    })();
  }, []);

  return (
    <Box flexDirection="row">
      <Box marginRight={1}>
        {state === StepState.IN_PROGRESS ? (
          <Text>
            [
            <Spinner type="dots" />
            ]
          </Text>
        ) : state === StepState.SUCCESS ? (
          <Text>
            [
            <Text color="green">✓</Text>
            ]
          </Text>
        ) : (
          <Text>
            [
            <Text color="red">×</Text>
            ]
          </Text>
        )}

        {state === StepState.FAIL ? (
          <Text>
            {title}
            <Newline />
            <Text color="red">{error!.message}</Text>
          </Text>
        ) : (
          <>
            <Text>{title}</Text>
          </>
        )}
      </Box>
    </Box>
  );
};

export default Task;
