import React from 'react';
import { proxy } from 'valtio';

import {
  ColumnsResultData,
  Named,
  TablesResultData,
} from 'cli/commands/migration/migrate';
import Task, { CallbackProps } from 'cli/components-api/components/Task';
import createApp from 'cli/components-api/CreateApp';
import { TableWithColumn } from 'cli/machines/resolveColumnsMachine';
import PromptTablesConflicts, { RenderTablesSelectProps } from 'cli/components-api/components/PromptTablesConflicts';
import PromptColumnsConflicts from 'cli/components-api/components/PromptColumnsConflicts';

const componentsList = proxy<React.ReactElement<any, any>[]>([]);
let app: ReturnType<typeof createApp>;

export interface CreateTaskParams<In, Cbs> {
  input: In,
  run: (cbs: Cbs) => void
}

const prepareApp = () => {
  if (!app) {
    app = createApp(componentsList);
    componentsList.length = 0;
  }
};

export const task = <T extends unknown>(
  titleStr: string,
  func: (props: CallbackProps) => T,
) => new Promise<T>((
    resolve,
  ) => {
    prepareApp();

    componentsList.push(
      <Task
        titleStr={titleStr}
        func={func}
        onDone={(val: T) => resolve(val)}
      />,
    );
  });

export const promptTablesConflicts = <T extends Named>(
  { missingTables, newTables }: RenderTablesSelectProps<T>,
) => new Promise<TablesResultData<T>>((
  resolve,
) => {
  prepareApp();

  componentsList.push(
    <PromptTablesConflicts
      onDone={(val) => resolve(val)}
      missingTables={missingTables}
      newTables={newTables}
    />,
  );
});

export const promptColumnsConflicts = (
  tableWithColumns: TableWithColumn,
) => new Promise<ColumnsResultData<Named>>((
  resolve,
) => {
  prepareApp();

  componentsList.push(
    <PromptColumnsConflicts
      onDone={(val) => resolve(val)}
      tableWithColumns={tableWithColumns}
    />,
  );
});
