import { fromJson } from "./sqlgenerator";
import resolveTables from "./simulator";
import { differ, diffForRenamed } from "./differ";
import fs from 'fs'
import { prepareAddValuesToEnumJson, prepareAlterTableColumnsJson, prepareCreateEnumJson, prepareCreateIndexesJson, prepareCreateTableJson, prepareDropIndexesJson, prepareDropTableJson, prepareRenameTableJson } from "./jsonStatements";

const json1 = {
  tables: {
    items: {
      name: "items",
      columns: {
        id: {
          name: "id",
          type: "int",
          primaryKey: true,
          autoincrement: true,
        },
        name: {
          name: "name",
          type: "character varying(256)",
          defaultValue: "Vetal",
          notNull: true,
        },
        unused: {
          name: "unused",
          type: "character varying(256)",
        },
        unused2: {
          name: "unused2",
          type: "character varying(256)",
        },
      },
      indexes: {
        index_to_be_deleted: {
          name: "index_to_be_deleted",
          columns: {
            phone: {
              name: "phone",
            },
          },
        },
      },
    },
    oldItems: {
      name: "old_items",
      columns: {},
    },
    items2: {
      name: "items2",
      columns: {},
    },
  },
  enums: {
    enum0: {
      name: "enum0",
      values: {
        value0: "value0",
        value1: "value1",
        value2: "value2",
      },
    },
    enum2: {
      name: "enum2",
      values: {
        value1: "value1",
        value2: "value2",
      },
    },
  },
};

const json2 = {
  tables: {
    items: {
      name: "items",
      columns: {
        id: {
          name: "id",
          type: "int",
          primaryKey: true,
          autoincrement: true,
        },
        name: {
          name: "name",
          type: "character varying(512)",
        },
        secondName: {
          name: "secondName",
          type: "character varying(512)",
          defaultValue: "Vitalik",
        },
        ownerId: {
          name: "owner_id",
          type: "int",
          notNull: true,
          defaultValue: 10,
        },
      },
      indexes: {
        users_phone_full_name_index: {
          name: "users_phone_full_name_index",
          columns: {
            phone: {
              name: "phone",
            },
            full_name: {
              name: "full_name",
            },
          },
        },
        users_phone_index: {
          name: "users_phone_index",
          columns: {
            phone: {
              name: "phone",
            },
          },
        },
      },
    },
    items3: {
      name: "items3",
      columns: {
        id: {
          name: "id",
          type: "int",
          primaryKey: true,
          autoincrement: true,
        },
        name: {
          name: "name",
          type: "character varying(512)",
          notNull: true,
          defaultValue: "defaultValue",
        },
        ownerId: {
          name: "owner_id",
          type: "int",
        },
      },
    },
    items22: {
      name: "items22",
      columns: {},
    },
  },
  enums: {
    enum0: {
      name: "enum0",
      values: {
        value1: "value1",
        value2: "value2",
        value3: "value3",
      },
    },
    enum1: {
      name: "enum1",
      values: {
        value1: "value1",
        value2: "value2",
      },
    },
  },
};

const dry = {
  version: "1",
  tables: {},
  enums: {}
}

// TODO: read snapshots from folder

// const j1 = JSON.parse(fs.readFileSync('./out/v1_1634546745488.json', 'utf8'))
const j1 = dry
const j2 = JSON.parse(fs.readFileSync('./out/v2_1634549601361.json', 'utf8'))
// const j3 = JSON.parse(fs.readFileSync('./out/1634215063491.json', 'utf8'))
const result = differ(j1, j2)

const { created, deleted, renamed } = resolveTables(result.addedTables, result.deletedTables)

const jsonStatements: any[] = []

const jsonCreateTables = created.map(it => {
  return prepareCreateTableJson(it)
})

const jsonCreateIndexesForCreatedTables = created.map(it => {
  return prepareCreateIndexesJson(it.name, it.indexes)
}).flat()

const jsonDropTables = deleted.map(it => {
  return prepareDropTableJson(it)
})

const jsonRenameTables = renamed.map(it => {
  return prepareRenameTableJson(it.from, it.to)
})

// const createNewTables = created.map(it => {
//   return prepareCreateTable(it)
// })
// console.log(createNewTables.join('\n').trim())

// const deleteTables = deleted.map(it => {
//   return prepareDropTable(it)
// })
// console.log(deleteTables.join('\n').trim())

const renamedWithAlternations = diffForRenamed(renamed)
const allAltered = result.alteredTablesWithColumns.concat(renamedWithAlternations)

// const renameTables = renamed.map(it => {
//   return prepareRenameTable(it.from, it.to)
// })
// console.log(renameTables.join('\n').trim())

const jsonAlterTables = allAltered.map(it => {
  return prepareAlterTableColumnsJson(it)
}).flat()

const jsonCreateIndexesForAllAlteredTables = allAltered.map(it => {
  return prepareCreateIndexesJson(it.name, it.addedIndexes || {})
}).flat()

const jsonDropIndexesForAllAlteredTables = allAltered.map(it => {
  return prepareDropIndexesJson(it.name, it.deletedIndexes || {})
}).flat()

// // Enums:
// // - создание енама ✅
// // - переименование енама (пока не делаю)⏳
// // - добавление вэлью к енаму ✅
// // - ренейм вейлью у енама (пока не делаю, это надо запрашивать опять же через слай)⏳
// // - удаление енама -> чекать не используется ли где-то енам и сначала ранить миграции и в самом конце удаление енама⏳
// // - удаление вэлью из енама -> блок ❌
// const enums = result.addedEnums.map(it => {
//   return prepareCreateEnum(it.name, it.values)
// })

const createEnums = result.addedEnums.map(it => {
  return prepareCreateEnumJson(it.name, it.values)
})

//todo: block enum rename, enum value rename and enun deletion for now
const jsonAlterEnumsWithAddedValues = result.alteredEnums.map(it => {
  return prepareAddValuesToEnumJson(it.name, it.addedValues)
}).flat()

jsonStatements.push(...createEnums)
jsonStatements.push(...jsonAlterEnumsWithAddedValues)
jsonStatements.push(...jsonCreateTables)
jsonStatements.push(...jsonCreateIndexesForCreatedTables)
jsonStatements.push(...jsonDropTables)
jsonStatements.push(...jsonRenameTables)
jsonStatements.push(...jsonAlterTables)
jsonStatements.push(...jsonCreateIndexesForAllAlteredTables)
jsonStatements.push(...jsonDropIndexesForAllAlteredTables)

// console.log(JSON.stringify(jsonStatements, null, 2))
console.log(jsonStatements)

const sqlStatements = fromJson(jsonStatements)
console.log(sqlStatements.join('\n'))


// console.log(enums.join('\n').trim())
// console.log(alteredEnums.join('\n').trim())


// explicitely ask if tables were renamed, if yes - add those to altered tables, otherwise - deleted
// double check if user wants to delete particular table and warn him on data loss
