'use-strict';

import { diff } from 'json-diff'


export function diffForRenamed(pairs) {
    // raname table1 to name of table2, so we can apply diffs
    const renamed = pairs.map(it => {
        const from = it.from || it.old
        const to = it.to || it.new
        const newFrom = { ...from, name: to.name }
        return [newFrom, to]
    })

    // find any alternations made to a renamed table
    const altered = renamed.map(pair => {
        const diffed = diff(pair[0], pair[1]);
        diffed.name = pair[0].name

        return findAlternationsInTable(diffed)
    })

    return altered
}
export function differ(json1, json2) {
    const difference = diff(json1, json2);

    difference.tables = difference.tables ? difference.tables : {}
    difference.enums = difference.enums ? difference.enums : {}

    const tableEntries = Object.entries(difference.tables)
    const addedTables = tableEntries.filter(it => it[0].includes('__added'))
        .map(it => it[1])
        .map(it => {
            // console.log(it)
            return {
                ...it, indexes: Object.entries(it.indexes).map(indexEntry => {
                    const idx = indexEntry[1]
                    const name = idx['name']
                    const columns = Object.values(idx['columns']).map(it => it['name'])
                    return { name, columns }
                })
            }
        })
    const deletedTables = tableEntries.filter(it => it[0].includes('__deleted')).map(it => it[1])

    const enumsEntries = Object.entries(difference.enums)

    const addedEnums = enumsEntries.filter(it => it[0].includes('__added'))
        .map(it => it[1])
        .map(it => {
            // values: { val1: 'val1', val2: 'val2' } => values: ['val1', 'val2']
            const values = Object.entries(it.values).map(ve => ve[1])
            return { name: it.name, values: values }
        })
    const deletedEnums = enumsEntries.filter(it => it[0].includes('__deleted'))
        .map(it => it[1])
        .map(it => {

            // values: { val1: 'val1', val2: 'val2' } => values: ['val1', 'val2']
            const values = Object.entries(it.values).map(ve => ve[1])
            return { name: it.name, values: values }
        })
    const alteredEnums = enumsEntries.filter(it => !(it[0].includes('__added') || it[0].includes('__deleted')))
        .map(it => {
            const vals = it[1].values
            const addedValues = Object.entries(vals).filter(val => val[0].includes('__added')).map(val => val[1])
            const deletedValues = Object.entries(vals).filter(val => val[0].includes('__deleted')).map(val => val[1])
            // console.log('values: ', it[1].values)
            return { name: it[0], addedValues, deletedValues, }
        })

    const alteredTables = Object.keys(difference.tables)
        .filter(it => !(it.includes('__added') || it.includes('__deleted')))
        .map(it => {
            return { name: it, ...difference.tables[it] }
        })

    const alteredTablesWithColumns = alteredTables.map(table => findAlternationsInTable(table))

    return {
        addedTables,
        deletedTables,
        alteredTablesWithColumns,
        addedEnums,
        deletedEnums,
        alteredEnums,
    }
}

const findAlternationsInTable = (table) => {
    // map each table to have altered, deleted or renamed columns
    const columns = table.columns; //in case no columns were altered, but indexes were

    const added = Object.keys(columns).filter(it => it.includes('__added')).map(it => {
        return { ...columns[it] }
    })
    const deleted = Object.keys(columns).filter(it => it.includes('__deleted')).map(it => {
        return { ...columns[it] }
    })
    const altered = Object.keys(columns)
        .filter(it => !(it.includes('__deleted') || it.includes('__added')))
        .map(it => {
            return { name: it, ...columns[it] }
        })

    const deletedIndexes = Object.values(table.indexes__deleted || {}).map(it => {
        const name = it['name']
        const columns = Object.values(it['columns']).map(it => it['name'])
        return { name, columns }
    }).concat(
        Object.keys(table.indexes || {}).filter(it => it.includes('__deleted'))
            .map(it => {
                const idx = table.indexes[it]
                const name = idx['name']
                const columns = Object.values(idx['columns']).map(it => it['name'])
                return { name, columns }
            })
    );

    const addedIndexes = Object.values(table.indexes__added || {}).map(it => {
        const name = it['name']
        const columns = Object.values(it['columns']).map(it => it['name'])
        return { name, columns }
    }).concat(
        Object.keys(table.indexes || {}).filter(it => it.includes('__added'))
            .map(it => {
                const idx = table.indexes[it]
                const name = idx['name']
                const columns = Object.values(idx['columns']).map(it => it['name'])
                return { name, columns }
            })
    );
    return { name: table.name, deleted, added, altered, addedIndexes, deletedIndexes }
}