declare global {
    interface Array<T> {
        exactlyOne(): T;
    }
}

Array.prototype.exactlyOne = function () {
    if (this.length !== 1) {
        return undefined
    }
    return this[0]
}

interface Table {
    name: string,
    indexes?: any
}

interface Handler {
    can(added: Table[], removed: Table[]): boolean
    handle(added: Table[], removed: Table[]): { created: Table[], deleted: Table[], renamed: { from: Table, to: Table }[] }
}

class DryRun implements Handler {
    can(_: Table[], removed: Table[]): boolean {
        return removed.length === 0
    }
    handle(added: Table[], _: Table[]): { created: Table[]; deleted: Table[]; renamed: { from: Table; to: Table; }[]; } {
        return { created: added, deleted: [], renamed: [] }
    }

}

// class Fallback implements Handler {
//     can(_: Table[], __: Table[]): boolean {
//         return true
//     }
//     handle(added: Table[], _: Table[]): { created: Table[]; deleted: Table[]; renamed: { from: Table; to: Table; }[]; } {
//         return { created: added, deleted: , renamed: [] }
//     }
// }

class Case1 implements Handler {
    can(_: Table[], removed: Table[]): boolean {
        return removed.length === 1 && removed[0].name === 'citiess'
    }

    handle(added: Table[], removed: Table[]): { created: Table[]; deleted: Table[]; renamed: { from: Table; to: Table; }[]; } {
        return { created: added, deleted: [], renamed: [{ from: removed[0], to: added[0] }] }
    }
}

class Case2 implements Handler {
    // authOtp, deleted, users -> authOtp renamed, cities added, deleted deleted
    can(_: Table[], removed: Table[]): boolean {
        return removed.length === 2 && removed[0].name === 'auth_otp'
    }

    handle(added: Table[], removed: Table[]): { created: Table[]; deleted: Table[]; renamed: { from: Table; to: Table; }[]; } {
        return { created: added.slice(1), deleted: removed.slice(1), renamed: [{ from: removed[0], to: added[0] }] }
    }
}

const handlers: Handler[] = []
handlers.push(new Case1())
handlers.push(new Case2())
handlers.push(new DryRun())


const resolveTables = (added: Table[], removed: Table[]) => {
    const handler = handlers.filter(it => {
        return it.can(added, removed)
    }).exactlyOne()

    if (!handler) {
        console.log('added', added.map(it => it.name).join())
        console.log('removed', removed.map(it => it.name).join())
        throw new Error("No handler");
    }

    console.log(`Simluated by ${handler.constructor.name}`)
    return handler.handle(added, removed)
}

export default resolveTables;
